/*:
 * @target MZ
 * @author Aerosys
 * @plugindesc [Tier 1] [Version 1.0.0] [MV & MZ]
 * 
 * @help
 * 
 * ----------------------------------------------------------------------------
 * Rules
 * ----------------------------------------------------------------------------
 * 1. This Plugin is free of charge and can be used in any kind of game.
 * 2. You may not redistribute this Plugin or claim it as your own.
 * 3. You may modify this plugin's source code for your needs but cannot
 *    redistribute your modifications.
 * 
 * 
 * NEED SUPPORT?
 * Contact me: mail<at>aerosys.blog
 * 
 * ----------------------------------------------------------------------------
 * Notetags
 * ----------------------------------------------------------------------------
 * 
 * - the following Notetags can be applied on Maps
 * 
 * <Minimap>
 * - shows the Minimap on this Map
 * 
 * <No Minimap>
 * - hides the Minimap on this Map
 * 
 * 
 * @command changeVisibility
 * @text Change Visibility
 * @desc Show or hide the Minimap
 * 
 * @arg mode
 * @text Mode
 * @type select
 * @option Show
 * @option Hide
 * @option Reset
 * @default Hide
 * 
 * 
 * @param common
 * @text Common
 * 
 * @param isVisible
 * @parent common
 * @text Visible by default?
 * @type boolean
 * @default true
 * @desc You can situationally show/hide the Minimap through Notetags and Plugin Commands
 * 
 * @param map
 * @parent common
 * @text Map Position & Appearance
 * @type struct<Map>
 * @default {"position":"top right","xEval":"\"const position = arguments[0];\\nconst width = arguments[1];\\n\\nreturn position == 'top left' || position == 'bottom left'\\n    ? (Graphics.width - Graphics.boxWidth) / 2\\n    : Graphics.width - ((Graphics.width - Graphics.boxWidth) / 2) - width\"","yEval":"\"const position = arguments[0];\\nconst height = arguments[1];\\n\\nlet result = position == 'top left' || position == 'top right'\\n    ? (Graphics.height - Graphics.boxHeight) / 2\\n    : Graphics.boxHeight + ((Graphics.height - Graphics.boxHeight) / 2) - height;\\n\\nif (position == 'top right' && this.buttonAreaHeight) {\\n    result = result + this.buttonAreaHeight();\\n}\\nreturn result;\"","widthEval":"Graphics.boxWidth * 0.3","heightEval":"Graphics.boxHeight * 0.3","opacity":"192"}
 * 
 * 
 * @param a
 * @text _
 * 
 * 
 * @param misc
 * @text Miscellaneous
 * 
 * @param addToOptions
 * @parent misc
 * @text Add to Options?
 * @type boolean
 * @default true
 * @desc Add Setting to the Options Menu
 * 
 * @param vocabulary
 * @parent misc
 * @text Vocabulary
 * @type struct<Vocabulary>
 * @default {"optionsMenuText":"Show Minimap"}
 */

/*~struct~Map:
 *
 * @param position
 * @text Position
 * @type select
 * @option top left
 * @option top right
 * @option bottom left
 * @option bottom right
 * @default top right
 * 
 * @param xEval
 * @text X Position
 * @type note
 * @default "const position = arguments[0];\nconst width = arguments[1];\n\nreturn position == 'top left' || position == 'bottom left'\n    ? (Graphics.width - Graphics.boxWidth) / 2\n    : Graphics.boxWidth - width + ((Graphics.width - Graphics.boxWidth) / 2);"
 * 
 * @param yEval
 * @text Y Position
 * @type note
 * @default "const position = arguments[0];\nconst height = arguments[1];\n\nlet result = position == 'top left' || position == 'top right'\n    ? (Graphics.height - Graphics.boxHeight) / 2\n    : Graphics.boxHeight + ((Graphics.height - Graphics.boxHeight) / 2) - height;\n\nif (position == 'top right' && this.buttonAreaHeight) {\n    result = result + this.buttonAreaHeight();\n}\nreturn result;"
 *
 * @param widthEval
 * @text Width
 * @desc You may use JavaScript
 * @default Graphics.boxWidth * 0.3
 * 
 * @param heightEval
 * @text Height
 * @desc You may use JavaScript
 * @default Graphics.boxHeight * 0.3
 * 
 * @param opacity
 * @text Opacity
 * @type number
 * @default 192
 * 
 */


/*~struct~Vocabulary:
 *
 * @param optionsMenuText
 * @text Options Menu: enable Minimap?
 * @default Show Minimap
 * 
 */


var MK = MK || { };
MK.Minimap = { };

(function() {

const customFunction = (body) => new Function(JSON.parse(body));


var params = PluginManager.parameters('MK_Minimap_Lite');

const mapParams                         = JSON.parse(params.map);
MK.Minimap.map                          = { }
MK.Minimap.isVisibleDefault             = 'true' == params.isVisible;
MK.Minimap.map.position                 = mapParams.position;
MK.Minimap.map.xEval                    = customFunction(mapParams.xEval);
MK.Minimap.map.yEval                    = customFunction(mapParams.yEval);
MK.Minimap.map.widthEval                = mapParams.widthEval;
MK.Minimap.map.heightEval               = mapParams.heightEval;
MK.Minimap.map.opacity                  = Number(mapParams.opacity);

MK.Minimap.addToOptions                 = 'true' == params.addToOptions;

const vocabularyParams                  = JSON.parse(params.vocabulary);
MK.Minimap.vocabulary                   = { };
MK.Minimap.vocabulary.optionsMenuText   = vocabularyParams.optionsMenuText;


// =====================================================================================
// Scene Boot
// =====================================================================================

const alias_SceneBoot_start = Scene_Boot.prototype.start;
Scene_Boot.prototype.start = function() {
    alias_SceneBoot_start.call(this);

    const reject = (message) => { throw Error(message); }

    const extractTier = (pluginDescription) => {
        const match = /\[Tier\s+\d+\]/g.exec(pluginDescription);
        if (match && match[0]) {
            const number = /\d+/.exec(match[0])[0];
            return Number(number);
        }
        return null;
    }

    const isListSorted = (list) => {
        if (list && list.length > 1) {
            for (let i = 1; i < list.length; i++) {
                if (list[i] < list[i - 1]) {
                    return false;
                }
            }
        }
        return true;
    }

    const getAllIndexesFromPlugins = (prefix) =>
        $plugins
            .filter(plugin => plugin && plugin.status)
            .map((plugin, index) => ({ plugin: plugin, index: index }))
            .filter(item => item.plugin.name.startsWith(prefix))
            .map(item => item.index);

    const mkTiers = $plugins
        .filter(plugin => plugin && plugin.status && plugin.name.startsWith('MK_'))
        .map(plugin => extractTier(plugin.description))
        .filter(tier => tier === 0 || tier > 0);
    
    if (!isListSorted(mkTiers)) {
        reject("MK Plugins are not in correct order. "
            + "Please go into the Plugin Manager and sort all the Plugins starting with \"MK\" "
            + "according to their tiers (Tier 0, Tier 1, ...)"
        );
    }
    
    const yepIndexes = getAllIndexesFromPlugins('YEP');
    const vsIndexes = getAllIndexesFromPlugins('VisuMZ');
    const mkIndexes = getAllIndexesFromPlugins('MK_');

    if (yepIndexes.length > 0 && Math.max(...yepIndexes) > Math.min(...mkIndexes)) {
        reject("All YEP Plugins must be above all MK Plugins");
    }
    
    if (vsIndexes.length > 0 && Math.max(...vsIndexes) > Math.min(...mkIndexes)) {
        reject("All Visustella Plugins must be above all MK Plugins, like this: VS Tier0, VS Tier1, VS Tier2, ..., MK Tier0, MK Tier1, MK Tier2, ...");
    }

    // set Tile Size
    MK.tileSize = $dataSystem.tileSize || 48;
}


// =====================================================================================
// Spriteset Minimap
// =====================================================================================

function Spriteset_Minimap() {
    this.initialize(...arguments);
}

if (typeof Sprite_Clickable !== 'undefined') {
    Spriteset_Minimap.prototype = Object.create(Sprite_Clickable.prototype);
    Spriteset_Minimap.prototype.constructor = Sprite_Clickable;
} else {
    Spriteset_Minimap.prototype = Object.create(Sprite.prototype);
    Spriteset_Minimap.prototype.constructor = Sprite;
}

Spriteset_Minimap.prototype.initialize = function(width, height) {
    if (typeof Sprite_Clickable !== 'undefined') {
        Sprite_Clickable.prototype.initialize.call(this);
    } else {
        Sprite.prototype.initialize.call(this);
    }

    this.width = width;
    this.height = height;
    
    this.createBackground();
    this.createMap();
    this.createEvents();
    this.update();
}

Spriteset_Minimap.prototype.createMap = function() {
    this.mapSprite = new Sprite_Minimap(
        $gameMap.width() * MK.tileSize,
        $gameMap.height() * MK.tileSize,
    );
    this.addChild(this.mapSprite);
}

Spriteset_Minimap.prototype.createEvents = function() {
    $gameMap.events()
        .map(event => new Sprite_Minimap_Event(event))
        .forEach(sprite => this.mapSprite.addChild(sprite));
    
    const boatSprite    = new Sprite_Minimap_Vehicle($gameMap.boat(), 'boat');
    const shipSprite    = new Sprite_Minimap_Vehicle($gameMap.ship(), 'ship');
    const airshipSprite = new Sprite_Minimap_Vehicle($gameMap.airship(), 'airship');
    const playerSprite  = new Sprite_Minimap_Player();
    
    this.mapSprite.addChild(boatSprite);
    this.mapSprite.addChild(shipSprite);
    this.mapSprite.addChild(airshipSprite);
    this.mapSprite.addChild(playerSprite);
}

Spriteset_Minimap.prototype.createBackground = function() {
    if ('MZ' == Utils.RPGMAKER_NAME) {
        const rectangle = new Rectangle(0, 0, this.width, this.height);
        this.background = new Window_Base(rectangle);
    } else {
        this.background = new Window_Base(0, 0, this.width, this.height);
    }
    this.addChild(this.background);
}

Spriteset_Minimap.prototype.update = function() {
    if (typeof Sprite_Clickable !== 'undefined') {
        Sprite_Clickable.prototype.update.call(this);
    } else {
        Sprite.prototype.update.call(this);
    }

    this.updateMain();
    this.updateMapScaleAndPosition();
}

Spriteset_Minimap.prototype.updateMain = function() {
    const scene = SceneManager._scene;
    const targetWidth = eval(MK.Minimap.map.widthEval);
    const targetHeight = eval(MK.Minimap.map.heightEval);
    
    if (isNaN(targetWidth)) rejectParameterValue('Minimap width', targetWidth);
    if (isNaN(targetHeight)) rejectParameterValue('Minimap height', targetHeight);

    const x = MK.Minimap.map.xEval.call(scene, MK.Minimap.map.position, targetWidth);
    const y = MK.Minimap.map.yEval.call(scene, MK.Minimap.map.position, targetHeight);

    if (isNaN(x)) rejectParameterValue('Minimap x', x);
    if (isNaN(y)) rejectParameterValue('Minimap y', y);
    
    this.move(x, y);
    this.width = targetWidth;
    this.height = targetHeight;
    this.opacity = MK.Minimap.map.opacity;
    this.visible = MK.Minimap.isVisible();
}

Spriteset_Minimap.prototype.updateMapScaleAndPosition = function() {
    const scale = Math.min(
        (this.width - (2 * this.padding())) / ($gameMap.width() * MK.tileSize),
        (this.height - (2 * this.padding())) / ($gameMap.height() * MK.tileSize),
    );
    this.mapSprite.scale.x = scale;
    this.mapSprite.scale.y = scale;

    const x = Math.floor(((this.width - (2 * this.padding()))
                - (this.mapSprite.width * this.mapSprite.scale.x)) / 2)
                + this.padding();
    const y = Math.floor(((this.height - (2 * this.padding()))
                - (this.mapSprite.height * this.mapSprite.scale.y)) / 2)
                + this.padding();
    this.mapSprite.move(x, y);
}

Spriteset_Minimap.prototype.padding = function() {
    if ('MZ' == Utils.RPGMAKER_NAME) {
        return $gameSystem.windowPadding();
    }
    else { // MV
        return Window_Base.prototype.standardPadding.call(this);
    }
}


// =====================================================================================
// Sprite Minimap
// =====================================================================================

function Sprite_Minimap() {
    this.initialize(...arguments);
}

Sprite_Minimap.prototype = Object.create(Sprite.prototype);
Sprite_Minimap.prototype.constructor = Sprite;

Sprite_Minimap.prototype.initialize = function(width, height) {
    const bitmap = new Bitmap(width, height);
    Sprite.prototype.initialize.call(this, bitmap);

    this.setup();
}

Sprite_Minimap.prototype.setup = function() {
    this.tilesetId = $gameMap.tilesetId();
    this.tilesetNames = $dataTilesets[this.tilesetId].tilesetNames;
    this.tilesetBitmaps = this.tilesetNames
        .map(name => name
            ? ImageManager.loadTileset(name)
            : null
        );
    
    for (let bitmap of this.tilesetBitmaps.filter(Boolean)) {
        bitmap.addLoadListener(this.onTilesetBitmapLoaded.bind(this));
    }
}

Sprite_Minimap.prototype.onTilesetBitmapLoaded = function() {
    if ($dataMap && this.isReady()) {
        this.createTilemap();
    }
}

Sprite_Minimap.prototype.createTilemap = function() {
    this.bitmap.clear();
    
    for (let x = 0; x < $dataMap.width; x++) {
        for (let y = 0; y < $dataMap.height; y++) {
            this.drawSpot(x, y);
        }
    }
}

Sprite_Minimap.prototype.drawSpot = function(x, y) {
    this.bitmap.clearRect(
        x * MK.tileSize,
        y * MK.tileSize,
        MK.tileSize,
        MK.tileSize,
    );
    for (let z = 0; z < 4; z++) {
        const tileId = $gameMap.tileId(x, y, z);

        if      (Tilemap.isTileA5(tileId))      this.drawTileA5(tileId, x, y);
        else if (Tilemap.isAutotile(tileId))    this.drawAutotile(tileId, x, y);
        else if (tileId)                        this.drawBTile(tileId, x, y);
    }
}

Sprite_Minimap.prototype.drawAutotile = function(tileId, x, y) {
    const w = MK.tileSize;
    const h = MK.tileSize;
    const w1 = w / 2;
    const h1 = h / 2;
    const kind = Tilemap.getAutotileKind(tileId);
    const shape = Tilemap.getAutotileShape(tileId);
    const tx = kind % 8;
    const ty = Math.floor(kind / 8);
    let autotileTable = Tilemap.FLOOR_AUTOTILE_TABLE;
    let tilesetNumber, bx, by;

    if (Tilemap.isTileA1(tileId)) {
        tilesetNumber = 0;
        const waterSurfaceIndex = 0;

        if (kind == 0) {
            bx = waterSurfaceIndex * 2;
            by = 0;
        }
        else if (kind == 1) {
            bx = waterSurfaceIndex * 2;
            by = 3;
        }
        else if (kind == 2) {
            bx = 6;
            by = 0;
        }
        else if (kind == 3) {
            bx = 6;
            by = 3;
        }
        else {
            bx = Math.floor(tx / 4) * 8;
            by = ty * 6 + (Math.floor(tx / 2) % 2) * 3;

            if (kind % 2 == 0) {
                bx += waterSurfaceIndex * 2;
            } else {
                bx += 6;
                autotileTable = Tilemap.WATERFALL_AUTOTILE_TABLE;
            }
        }
    }
    if (Tilemap.isTileA2(tileId)) {
        tilesetNumber = 1;
        bx = tx * 2;
        by = (ty - 2) * 3;
    }
    if (Tilemap.isTileA3(tileId)) {
        tilesetNumber = 2;
        bx = tx * 2;
        by = (ty - 6) * 2;
        autotileTable = Tilemap.WALL_AUTOTILE_TABLE;
    }
    if (Tilemap.isTileA4(tileId)) {
        tilesetNumber = 3;
        bx = tx * 2;
        by = Math.floor((ty - 10) * 2.5 + (ty % 2 === 1 ? 0.5 : 0));
        if (ty % 2 === 1) {
            autotileTable = Tilemap.WALL_AUTOTILE_TABLE;
        }
    }
    
    for (let i = 0; i < 4; i++) {
        const qsx = autotileTable[shape][i][0];
        const qsy = autotileTable[shape][i][1];
        const sx  = (bx * 2 + qsx) * w1;
        const sy  = (by * 2 + qsy) * h1;
        const dx  = (i % 2) * w1 + (w * x);
        const dy  = Math.floor(i / 2) * h1 + (h * y);
        
        this.blt(
            tilesetNumber,
            sx,
            sy,
            w1,
            h1,
            dx,
            dy,
        );
    }
}

Sprite_Minimap.prototype.drawTileA5 = function(tileId, x, y) {
    this.drawNormalTile(4, tileId, x, y);
}

Sprite_Minimap.prototype.drawBTile = function(tileId, x, y) {
    const tilesetNumber = Math.floor(tileId / 256) + 5;
    this.drawNormalTile(tilesetNumber, tileId, x, y);
}

Sprite_Minimap.prototype.drawNormalTile = function(tilesetNumber, tileId, x, y) {
    const w = MK.tileSize;
    const h = MK.tileSize;
    const sx = ((Math.floor(tileId / 128) % 2) * 8 + (tileId % 8)) * w;
    const sy = (Math.floor((tileId % 256) / 8) % 16) * h;
    
    this.blt(
        tilesetNumber,
        sx,
        sy,
        w,
        h,
        w * x,
        h * y,
    );
}

Sprite_Minimap.prototype.blt = function(tilesetNumber, sx, sy, w, h, dx, dy) {
    const tilesetBitmap = this.tilesetBitmaps[tilesetNumber];

    this.bitmap.blt(
        tilesetBitmap,
        sx,
        sy,
        w,
        h,
        dx,
        dy,
    );
}

Sprite_Minimap.prototype.isReady = function() {
    return this.tilesetBitmaps
        && this.tilesetBitmaps.every(bitmap => !bitmap || bitmap.isReady());
}


// =====================================================================================
// Sprite Character
// =====================================================================================

function Sprite_Minimap_Character() {
    this.initialize(...arguments);
}

Sprite_Minimap_Character.prototype = Object.create(Sprite.prototype);
Sprite_Minimap_Character.prototype.constructor = Sprite;

Sprite_Minimap_Character.prototype.initialize = function(event) {
    this.event = event;
    Sprite.prototype.initialize.call(this, new Bitmap());

    this.anchor.x = 0.5;
    this.anchor.y = 1.0;
}

Sprite_Minimap_Character.prototype.update = function() {
    Sprite.prototype.update.call(this);

    this.updateModeIfRequired();
    this.updateFrame();
    this.updatePosition();
    
    this.visible = this.isVisible();
    this.opacity = this.getOpacity();
    this.blendMode = this.event.blendMode();
}

Sprite_Minimap_Character.prototype.updateModeIfRequired = function() {
    if (this.requiresRefresh()) {
        this.tilesetId      = $gameMap.tilesetId();
        this.tileId         = this.event.tileId();
        this.characterName  = this.event.characterName();
        this.characterIndex = this.event.characterIndex();
        
        this.updateBitmap();
    }
}

Sprite_Minimap_Character.prototype.requiresRefresh = function() {
    return this.tilesetId != $gameMap.tilesetId()
        || this.tileId != this.event.tileId()
        || this.characterName != this.event.characterName()
        || this.characterIndex != this.event.characterIndex();
}

Sprite_Minimap_Character.prototype.updateBitmap = function() {
    if (this.characterName) {
        this.bitmap = ImageManager.loadCharacter(this.characterName);
        this.isBigCharacter = ImageManager.isBigCharacter(this.characterName);
    }
    else if (this.tileId) {
        const tileset       = $gameMap.tileset();
        const setNumber     = Math.floor(this.tileId / 256) + 5;
        const tilesetName   = tileset.tilesetNames[setNumber];
        this.bitmap         = ImageManager.loadTileset(tilesetName);
    }
    else {
        this.bitmap = new Bitmap();
    }
}

Sprite_Minimap_Character.prototype.updateFrame = function() {
    if (this.tileId) {
        const pw = MK.tileSize;
        const ph = MK.tileSize;
        const sx = ((Math.floor(this.tileId / 128) % 2) * 8 + (this.tileId % 8)) * pw;
        const sy = (Math.floor((this.tileId % 256) / 8) % 16) * ph;
        this.setFrame(sx, sy, pw, ph);
    }
    else if (this.characterName) {
        const pw = this.patternWidth();
        const ph = this.patternHeight();
        const sx = (this.characterBlockX() + this.characterPatternX()) * pw;
        const sy = (this.characterBlockY() + this.characterPatternY()) * ph;
        this.setFrame(sx, sy, pw, ph);
    }
}

Sprite_Minimap_Character.prototype.updatePosition = function() {
    this.move(
        (this.event._realX + 0.5) * MK.tileSize,
        (this.event._realY + this.anchor.y) * MK.tileSize - this.event.shiftY(),
    );
}

Sprite_Minimap_Character.prototype.patternWidth = function() {
    return this.bitmap.width / (this.isBigCharacter ? 3 : 12);
}

Sprite_Minimap_Character.prototype.patternHeight = function() {
    return this.bitmap.height / (this.isBigCharacter ? 4 : 8);
}

Sprite_Minimap_Character.prototype.characterPatternX = function() {
    return this.event.pattern();
}

Sprite_Minimap_Character.prototype.characterPatternY = function() {
    return (this.event.direction() - 2) / 2;
}

Sprite_Minimap_Character.prototype.characterBlockX = function() {
    return this.isBigCharacter
        ? 0
        : Math.floor(this.event.characterIndex() % 4) * 3;
}

Sprite_Minimap_Character.prototype.characterBlockY = function() {
    return this.isBigCharacter
        ? 0
        : Math.floor(this.event.characterIndex() / 4) * 4;
}

Sprite_Minimap_Character.prototype.getOpacity = function() {
    return 255;
}

Sprite_Minimap_Character.prototype.isVisible = function() {
    return true;
}


// =====================================================================================
// Sprite Event
// =====================================================================================

function Sprite_Minimap_Event() {
    this.initialize(...arguments);
}

Sprite_Minimap_Event.prototype = Object.create(Sprite_Minimap_Character.prototype);
Sprite_Minimap_Event.prototype.constructor = Sprite_Minimap_Character;

Sprite_Minimap_Event.prototype.update = function() {
    Sprite_Minimap_Character.prototype.update.call(this);

    this.pageIndex = this.event._pageIndex;
}

Sprite_Minimap_Event.prototype.requiresRefresh = function() {
    return (
           Sprite_Minimap_Character.prototype.requiresRefresh.call(this)
        || this.pageIndex !== this.event._pageIndex
    );
}

Sprite_Minimap_Event.prototype.isVisible = function() {
    return (
           Sprite_Minimap_Character.prototype.isVisible.call(this)
        && !(this.event.isTransparent && this.event.isTransparent())
    );
}


// =====================================================================================
// Sprite Player
// =====================================================================================

function Sprite_Minimap_Player() {
    this.initialize(...arguments);
}

Sprite_Minimap_Player.prototype = Object.create(Sprite_Minimap_Character.prototype);
Sprite_Minimap_Player.prototype.constructor = Sprite_Minimap_Character;

Sprite_Minimap_Player.prototype.initialize = function() {
    Sprite_Minimap_Character.prototype.initialize.call(
        this,
        $gamePlayer,
    );
}

Sprite_Minimap_Player.prototype.isVisible = function() {
    return (
           Sprite_Minimap_Character.prototype.isVisible.call(this)
        && !$gamePlayer.vehicle()
    )
}


// =====================================================================================
// Sprite Vehicle
// =====================================================================================

function Sprite_Minimap_Vehicle() {
    this.initialize(...arguments);
}

Sprite_Minimap_Vehicle.prototype = Object.create(Sprite_Minimap_Character.prototype);
Sprite_Minimap_Vehicle.prototype.constructor = Sprite_Minimap_Character;

Sprite_Minimap_Vehicle.prototype.initialize = function(vehicle, type) {
    this.type = type;
    
    Sprite_Minimap_Character.prototype.initialize.call(
        this,
        vehicle,
    );
}

Sprite_Minimap_Vehicle.prototype.isVisible = function() {
    return (
           Sprite_Minimap_Character.prototype.isVisible.call(this)
        && this.event._mapId == $gameMap.mapId()
    );
}


// =====================================================================================
// Scene Map
// =====================================================================================

const rejectParameterValue = (param, value) => {
    const message = "MK Minimap: The parameter '%1' did not return a valid value, but %2".format(param, value);
    throw Error(message);
}


const alias_SceneMap_createDisplayObjects = Scene_Map.prototype.createDisplayObjects;
Scene_Map.prototype.createDisplayObjects = function() {
    alias_SceneMap_createDisplayObjects.call(this);
    
    this.createMinimap();
}

Scene_Map.prototype.createMinimap = function() {
    const targetWidth = eval(MK.Minimap.map.widthEval);
    const targetHeight = eval(MK.Minimap.map.heightEval);
    if (isNaN(targetWidth)) rejectParameterValue('Minimap width', targetWidth);
    if (isNaN(targetHeight)) rejectParameterValue('Minimap height', targetHeight);
    
    this._minimapSprite = new Spriteset_Minimap(targetWidth, targetHeight);
    this.addChild(this._minimapSprite);
}


// =====================================================================================
// MK
// =====================================================================================

MK.Minimap.isVisible = function() {
    if (!$dataMap)                      return false;
    if ($gameSystem.hideMinimap)        return false;
    if ($gameSystem.showMinimap)        return true;
    if (!ConfigManager.showMinimap)     return false;
    if ($dataMap.meta['Minimap'])       return true;
    if ($dataMap.meta['No Minimap'])    return false;

    return MK.Minimap.isVisibleDefault;
}

MK.Minimap.show = function() {
    $gameSystem.showMinimap = true;
    $gameSystem.hideMinimap = false;
}

MK.Minimap.hide = function() {
    $gameSystem.showMinimap = false;
    $gameSystem.hideMinimap = true;
}

MK.Minimap.resetVisibility = function() {
    $gameSystem.showMinimap = false;
    $gameSystem.hideMinimap = false;
}


// =====================================================================================
// Config Manager
// =====================================================================================

ConfigManager.showMinimap = true;

if (MK.Minimap.addToOptions) {
    
    const alias_ConfigManager_makeData = ConfigManager.makeData;
    ConfigManager.makeData = function() {
        const config = alias_ConfigManager_makeData.call(this);
        config.showMinimap = this.showMinimap;
        return config;
    }

    const alias_ConfigManager_applyData = ConfigManager.applyData;
    ConfigManager.applyData = function(config) {
        alias_ConfigManager_applyData.call(this, config);
        
        this.showMinimap = config.showMinimap || !('showMinimap' in config);
    }

    const alias_WindowOptions_addGeneralOptions = Window_Options.prototype.addGeneralOptions;
    Window_Options.prototype.addGeneralOptions = function() {
        alias_WindowOptions_addGeneralOptions.call(this);

        this.addCommand(MK.Minimap.vocabulary.optionsMenuText, 'showMinimap');
    }

    const alias_SceneOptions_maxCommands = Scene_Options.prototype.maxCommands;
    Scene_Options.prototype.maxCommands = function() {
        return alias_SceneOptions_maxCommands.call(this) + 1;
    }
}


// =====================================================================================
// Plugin Manager
// =====================================================================================

if (PluginManager && PluginManager.registerCommand) {

    PluginManager.registerCommand('MK_Minimap_Lite', 'changeVisibility', args => {
        $gameSystem.showMinimap = 'Show' == args.mode;
        $gameSystem.hideMinimap = 'Hide' == args.mode;
    });
}


})();